@@ -1,19 +1,25 @@
+'''
+Checks for the Attribute Conventions for Dataset Discovery (ACDD)
+
+This module contains classes defined as checks part of the compliance checker
+project for the verification and scoring of attributes for datasets.
+'''
+
 from __future__ import unicode_literals
-import itertools
+
 import numpy as np
 import numpy.ma as ma
-
-from dateutil.parser import parse as parse_dt
-from cf_units import Unit
+from netCDF4 import num2date
+from datetime import timedelta
 
 from compliance_checker.base import (BaseCheck, BaseNCCheck, check_has,
                                      score_group, Result, ratable_result)
-from compliance_checker.cf.util import (is_time_variable,
-                                        is_vertical_coordinate,
-                                       _possiblexunits, _possibleyunits)
-from compliance_checker.util import datetime_is_iso
+from compliance_checker.cf.util import is_vertical_coordinate, _possiblexunits, _possibleyunits
+from compliance_checker.util import datetime_is_iso, dateparse
+from compliance_checker import cfutil
 from pygeoif import from_wkt
 
+
 class ACDDBaseCheck(BaseCheck):
 
     _cc_spec = 'acdd'
@@ -21,9 +27,12 @@ class ACDDBaseCheck(BaseCheck):
     _cc_url = 'http://wiki.esipfed.org/index.php?title=Category:Attribute_Conventions_Dataset_Discovery'
 
     def __init__(self):
-        self.high_rec_atts = ['title',
-                              'keywords',
-                              'summary']
+
+        self.high_rec_atts = [
+            'title',
+            'keywords',
+            'summary'
+        ]
 
         self.rec_atts = [
             'id',
@@ -37,8 +46,6 @@ class ACDDBaseCheck(BaseCheck):
             'institution',
             'project',
             'processing_level',
-            # mutliple spellings get tested, move to check fn instead
-            #'acknowledgement',
             ('geospatial_bounds', self.verify_geospatial_bounds),
             'geospatial_lat_min',
             'geospatial_lat_max',
@@ -67,70 +74,71 @@ class ACDDBaseCheck(BaseCheck):
             'geospatial_vertical_resolution'
         ]
 
-    ###############################################################################
-    #
-    # HIGHLY RECOMMENDED ATTRIBUTES
-    #
-    ###############################################################################
+        # This variable is used to cache the results of applicable variables so
+        # the method isn't executed repeatedly.
+        self._applicable_variables = None
+
     # set up attributes according to version
     @check_has(BaseCheck.HIGH)
     def check_high(self, ds):
+        '''
+        Performs a check on each highly recommended attributes' existence in the dataset
+
+        :param netCDF4.Dataset ds: An open netCDF dataset
+        '''
         return self.high_rec_atts
-    ###############################################################################
-    #
-    # RECOMMENDED ATTRIBUTES
-    #
-    ###############################################################################
 
     @check_has(BaseCheck.MEDIUM)
     def check_recommended(self, ds):
+        '''
+        Performs a check on each recommended attributes' existence in the dataset
+
+        :param netCDF4.Dataset ds: An open netCDF dataset
+        '''
         return self.rec_atts
-    ###############################################################################
-    #
-    # SUGGESTED ATTRIBUTES
-    #
-    ###############################################################################
 
     @check_has(BaseCheck.LOW)
     def check_suggested(self, ds):
         return self.sug_atts
 
-    ###############################################################################
-    #
-    # HIGHLY RECOMMENDED VARIABLE ATTRS
-    #
-    ###############################################################################
+    def get_applicable_variables(self, ds):
+        '''
+        Returns a list of variable names that are applicable to ACDD Metadata
+        Checks for variables. This includes geophysical and coordinate
+        variables only.
 
-    def _get_vars(self, ds, attr_filter=None):
-        vars = ds.dogma._eval_xpath('//ncml:variable')
-
-        if attr_filter is not None:
-            attrs = itertools.chain.from_iterable((v.xpath('ncml:attribute[@name="%s"]/@value' % attr_filter, namespaces=ds.dogma._namespaces) or [None] for v in vars))
-            names = (v.get('name', 'unknown') for v in vars)
-
-            attrs = list(zip(attrs, names))
-
-            return attrs
-
-        return vars
-
-    def _get_msg(self, vpair, attr):
-        vval, vname = vpair
-        if vval is None:
-            return ["Var %s missing attr %s" % (vname, attr)]
-
-        return []
+        :param netCDF4.Dataset ds: An open netCDF dataset
+        '''
+        if self._applicable_variables is None:
+            self.applicable_variables = cfutil.get_geophysical_variables(ds)
+            varname = cfutil.get_time_variable(ds)
+            if varname:
+                self.applicable_variables.append(varname)
+            varname = cfutil.get_lon_variable(ds)
+            if varname:
+                self.applicable_variables.append(varname)
+            varname = cfutil.get_lat_variable(ds)
+            if varname:
+                self.applicable_variables.append(varname)
+            varname = cfutil.get_z_variable(ds)
+            if varname:
+                self.applicable_variables.append(varname)
+        return self.applicable_variables
 
     @score_group('varattr')
     def check_var_long_name(self, ds):
+        '''
+        Checks each applicable variable for the long_name attribute
+
+        :param netCDF4.Dataset ds: An open netCDF dataset
+        '''
         results = []
-        # We don't check certain container variables for units
-        platform_variable_name = getattr(ds, 'platform', None)
-        for variable in ds.variables:
+
+        # ACDD Variable Metadata applies to all coordinate variables and
+        # geophysical variables only.
+
+        for variable in self.get_applicable_variables(ds):
             msgs = []
-            if variable in ('crs', platform_variable_name):
-                continue
-            # If the variable is a QC flag, we don't need units
             long_name = getattr(ds.variables[variable], 'long_name', None)
             check = long_name is not None
             if not check:
@@ -141,14 +149,14 @@ class ACDDBaseCheck(BaseCheck):
 
     @score_group('varattr')
     def check_var_standard_name(self, ds):
+        '''
+        Checks each applicable variable for the standard_name attribute
+
+        :param netCDF4.Dataset ds: An open netCDF dataset
+        '''
         results = []
-        # We don't check certain container variables for units
-        platform_variable_name = getattr(ds, 'platform', None)
-        for variable in ds.variables:
+        for variable in self.get_applicable_variables(ds):
             msgs = []
-            if variable in ('crs', platform_variable_name):
-                continue
-            # If the variable is a QC flag, we don't need units
             std_name = getattr(ds.variables[variable], 'standard_name', None)
             check = std_name is not None
             if not check:
@@ -157,21 +165,16 @@ class ACDDBaseCheck(BaseCheck):
 
         return results
 
-
     @score_group('varattr')
     def check_var_units(self, ds):
+        '''
+        Checks each applicable variable for the units attribute
+
+        :param netCDF4.Dataset ds: An open netCDF dataset
+        '''
         results = []
-        # We don't check certain container variables for units
-        platform_variable_name = getattr(ds, 'platform', None)
-        for variable in ds.variables:
+        for variable in self.get_applicable_variables(ds):
             msgs = []
-            if variable in ('crs', platform_variable_name):
-                continue
-            # If the variable is a QC flag, we don't need units
-            std_name = getattr(ds.variables[variable], 'standard_name', None)
-            if std_name is not None:
-                if 'status_flag' in std_name:
-                    continue
             # Check units and dims for variable
             unit_check = hasattr(ds.variables[variable], 'units')
             no_dim_check = (getattr(ds.variables[variable], 'dimensions') == tuple())
@@ -185,49 +188,28 @@ class ACDDBaseCheck(BaseCheck):
 
         return results
 
-    ################################################################################
-    #
-    # HIGHLY RECOMMENDED CHECKS
-    #
-    ###############################################################################
-
-
     def check_acknowledgment(self, ds):
-        """Check if acknowledgment/acknowledgment attr is present"""
-        if not (hasattr(ds, 'acknowledgment') or
-                hasattr(ds, 'acknowledgement')):
-            return Result(BaseCheck.MEDIUM, False,
-                          'acknowledgment/acknowledgement', msgs = [])
+        '''
+        Check if acknowledgment/acknowledgment attr is present.
+
+        :param netCDF4.Dataset ds: An open netCDF dataset
+        '''
+        check = False
+        messages = []
+        if hasattr(ds, 'acknowledgment') or hasattr(ds, 'acknowledgement'):
+            check = True
         else:
-            return Result(BaseCheck.HIGH, True,
-                          'acknowledgment/acknowledgement',
-                          msgs=["Neither 'acknowledgment' nor 'acknowledgement' attributes present"])
+            messages.append("acknowledgement global attribute is recommended")
 
-
-    ###############################################################################
-    #
-    # RECOMMENDED CHECKS
-    #
-    ###############################################################################
-
-
-    ###############################################################################
-    #
-    # SUGGESTED
-    #
-    ###############################################################################
-
-
-    ###############################################################################
-    #
-    # DATA EXTENTS MATCHING ATTRIBUTES
-    #
-    ###############################################################################
+        return Result(BaseCheck.MEDIUM, check, 'acknowledgment/acknowledgement', msgs=messages)
 
     def check_lat_extents(self, ds):
-        """
-        Check that the values of geospatial_lat_min/geospatial_lat_max approximately match the data.
-        """
+        '''
+        Check that the values of geospatial_lat_min/geospatial_lat_max
+        approximately match the data.
+
+        :param netCDF4.Dataset ds: An open netCDF dataset
+        '''
         if not (hasattr(ds, 'geospatial_lat_min') and hasattr(ds, 'geospatial_lat_max')):
             return
 
@@ -288,9 +270,12 @@ class ACDDBaseCheck(BaseCheck):
                       msgs)
 
     def check_lon_extents(self, ds):
-        """
-        Check that the values of geospatial_lon_min/geospatial_lon_max approximately match the data.
-        """
+        '''
+        Check that the values of geospatial_lon_min/geospatial_lon_max
+        approximately match the data.
+
+        :param netCDF4.Dataset ds: An open netCDF dataset
+        '''
         if not (hasattr(ds, 'geospatial_lon_min') and hasattr(ds, 'geospatial_lon_max')):
             return
 
@@ -356,8 +341,8 @@ class ACDDBaseCheck(BaseCheck):
         check = var is not None
         if not check:
             return ratable_result(False,
-                          'geospatial_bounds',
-                          ["Attr geospatial_bounds not present"])
+                                  'geospatial_bounds',
+                                  ["Attr geospatial_bounds not present"])
 
         try:
             # TODO: verify that WKT is valid given CRS (defaults to EPSG:4326
@@ -365,52 +350,101 @@ class ACDDBaseCheck(BaseCheck):
             from_wkt(ds.geospatial_bounds)
         except AttributeError:
             return ratable_result(False,
-                          'geospatial_bounds',
-                          ['Could not parse WKT, possible bad value for WKT'])
+                                  'geospatial_bounds',
+                                  ['Could not parse WKT, possible bad value for WKT'])
         # parsed OK
         else:
-            return ratable_result(True, 'geospatial_bounds',
-                          ())
+            return ratable_result(True, 'geospatial_bounds', tuple())
 
+    def _check_total_z_extents(self, ds, z_variable):
+        '''
+        Check the entire array of Z for minimum and maximum and compare that to
+        the vertical extents defined in the global attributes
+
+        :param netCDF4.Dataset ds: An open netCDF dataset
+        :param str z_variable: Name of the variable representing the Z-Axis
+        '''
+        vert_min = ds.geospatial_vertical_min
+        vert_max = ds.geospatial_vertical_max
+        msgs = []
+        total = 2
+
+        zvalue = ds.variables[z_variable][:]
+        # If the array has fill values, which is allowed in the case of point
+        # features
+        if hasattr(zvalue, 'mask'):
+            zvalue = zvalue[~zvalue.mask]
+        zmin = zvalue.min()
+        zmax = zvalue.max()
+        if not np.isclose(vert_min, zmin):
+            msgs.append("geospatial_vertical_min != min(%s) values, %s != %s" % (
+                z_variable,
+                vert_min,
+                zmin
+            ))
+        if not np.isclose(vert_max, zmax):
+            msgs.append("geospatial_vertical_max != max(%s) values, %s != %s" % (
+                z_variable,
+                vert_min,
+                zmax
+            ))
+
+        return Result(BaseCheck.MEDIUM,
+                      (total - len(msgs), total),
+                      'geospatial_vertical_extents_match',
+                      msgs)
+
+    def _check_scalar_vertical_extents(self, ds, z_variable):
+        '''
+        Check the scalar value of Z compared to the vertical extents which
+        should also be equivalent
+
+        :param netCDF4.Dataset ds: An open netCDF dataset
+        :param str z_variable: Name of the variable representing the Z-Axis
+        '''
+        vert_min = ds.geospatial_vertical_min
+        vert_max = ds.geospatial_vertical_max
+        msgs = []
+        total = 2
+
+        zvalue = ds.variables[z_variable][:].item()
+        if not np.isclose(vert_min, vert_max):
+            msgs.append("geospatial_vertical_min != geospatial_vertical_max for scalar depth values, %s != %s" % (
+                vert_min,
+                vert_max
+            ))
+
+        if not np.isclose(vert_max, zvalue):
+            msgs.append("geospatial_vertical_max != %s values, %s != %s" % (
+                z_variable,
+                vert_max,
+                zvalue
+            ))
+
+        return Result(BaseCheck.MEDIUM,
+                      (total - len(msgs), total),
+                      'geospatial_vertical_extents_match',
+                      msgs)
 
     def check_vertical_extents(self, ds):
         """
         Check that the values of geospatial_vertical_min/geospatial_vertical_max approximately match the data.
+
+        :param netCDF4.Dataset ds: An open netCDF dataset
         """
         if not (hasattr(ds, 'geospatial_vertical_min') and hasattr(ds, 'geospatial_vertical_max')):
             return
 
-        vert_min = ds.geospatial_vertical_min
-        vert_max = ds.geospatial_vertical_max
-
-        # identify vertical vars as per CF 4.3
-        v_vars = [(var._name, ma.array(var)) for name, var in
-                  ds.variables.items() if is_vertical_coordinate(name, var)]
-
-        if len(v_vars) == 0:
+        z_variable = cfutil.get_z_variable(ds)
+        if not z_variable:
             return Result(BaseCheck.MEDIUM,
                           False,
                           'geospatial_vertical_extents_match',
                           ['Could not find vertical variable to test extent of geospatial_vertical_min/geospatial_vertical_max, see CF-1.6 spec chapter 4.3'])
+        if ds.variables[z_variable].dimensions == tuple():
+            return self._check_scalar_vertical_extents(ds, z_variable)
 
-        obs_mins = {var[0]: np.nanmin(var[1]) for var in v_vars if not np.isnan(var[1]).all()}
-        obs_maxs = {var[0]: np.nanmax(var[1]) for var in v_vars if not np.isnan(var[1]).all()}
-
-        min_pass = any((np.isclose(vert_min, min_val) for min_val in obs_mins.values()))
-        max_pass = any((np.isclose(vert_max, max_val) for max_val in obs_maxs.values()))
-
-        allpass = sum((min_pass, max_pass))
-
-        msgs = []
-        if not min_pass:
-            msgs.append("Data for possible vertical variables (%s) did not match geospatial_vertical_min value (%s)" % (obs_mins, vert_min))
-        if not max_pass:
-            msgs.append("Data for possible vertical variables (%s) did not match geospatial_vertical_max value (%s)" % (obs_maxs, vert_max))
-
-        return Result(BaseCheck.MEDIUM,
-                      (allpass, 2),
-                      'geospatial_vertical_extents_match',
-                      msgs)
+        return self._check_total_z_extents(ds, z_variable)
 
     def check_time_extents(self, ds):
         """
@@ -420,40 +454,49 @@ class ACDDBaseCheck(BaseCheck):
             return
 
         # allows non-ISO 8601 formatted dates
-        epoch = parse_dt("1970-01-01 00:00:00 UTC")
         try:
-            t_min = (parse_dt(ds.time_coverage_start) - epoch).total_seconds()
-            t_max = (parse_dt(ds.time_coverage_end) - epoch).total_seconds()
+            t_min = dateparse(ds.time_coverage_start)
+            t_max = dateparse(ds.time_coverage_end)
         except:
             return Result(BaseCheck.MEDIUM,
                           False,
                           'time_coverage_extents_match',
-                          ['time_coverage variables are not formatted properly'])
-        # identify t vars as per CF 4.4
-        t_vars = [var for name, var in ds.variables.items() if is_time_variable(name, var)]
+                          ['time_coverage variables are not formatted properly. Please ensure they are valid ISO-8601 time strings'])
 
-        if len(t_vars) == 0:
+        timevar = cfutil.get_time_variable(ds)
+
+        if not timevar:
             return Result(BaseCheck.MEDIUM,
                           False,
                           'time_coverage_extents_match',
                           ['Could not find time variable to test extent of time_coverage_start/time_coverage_end, see CF-1.6 spec chapter 4.4'])
 
-        obs_mins = {var._name: Unit(str(var.units)).convert(np.nanmin(var), "seconds since 1970-01-01") for var in t_vars}
-        obs_maxs = {var._name: Unit(str(var.units)).convert(np.nanmax(var), "seconds since 1970-01-01") for var in t_vars}
+        # Time should be monotonically increasing, so we make that assumption here so we don't have to download THE ENTIRE ARRAY
+        try:
+            time0 = num2date(ds.variables[timevar][0], ds.variables[timevar].units)
+            time1 = num2date(ds.variables[timevar][-1], ds.variables[timevar].units)
+        except:
+            return Result(BaseCheck.MEDIUM,
+                          False,
+                          'time_coverage_extents_match',
+                          ['Failed to retrieve and convert times for variables %s.' % timevar])
 
-        min_pass = any((np.isclose(t_min, min_val) for min_val in obs_mins.values()))
-        max_pass = any((np.isclose(t_max, max_val) for max_val in obs_maxs.values()))
-
-        allpass = sum((min_pass, max_pass))
+        start_dt = abs(time0 - t_min)
+        end_dt = abs(time1 - t_max)
 
+        score = 2
         msgs = []
-        if not min_pass:
-            msgs.append("Data for possible time variables (%s) did not match time_coverage_start value (%s)" % (obs_mins, t_min))
-        if not max_pass:
-            msgs.append("Data for possible time variables (%s) did not match time_coverage_end value (%s)" % (obs_maxs, t_max))
+        if start_dt > timedelta(hours=1):
+            msgs.append("Date time mismatch between time_coverage_start and actual "
+                        "time values %s (time_coverage_start) != %s (time[0])" % (t_min.isoformat(), time0.isoformat()))
+            score -= 1
+        if end_dt > timedelta(hours=1):
+            msgs.append("Date time mismatch between time_coverage_end and actual "
+                        "time values %s (time_coverage_end) != %s (time[N])" % (t_max.isoformat(), time1.isoformat()))
+            score -= 1
 
         return Result(BaseCheck.MEDIUM,
-                      (allpass, 2),
+                      (score, 2),
                       'time_coverage_extents_match',
                       msgs)
 
@@ -461,24 +504,25 @@ class ACDDBaseCheck(BaseCheck):
         """
         Verify that the version in the Conventions field is correct
         """
-        for convention in ds.Conventions.replace(' ','').split(','):
+        for convention in ds.Conventions.replace(' ', '').split(','):
             if convention == 'ACDD-' + self._cc_spec_version:
-                return ratable_result(
-                        (2,2),
-                        'Conventions',
-                        [])
+                return ratable_result((2, 2),
+                                      'Conventions',
+                                      [])
         # Conventions attribute is present, but does not include
         # proper ACDD version
-        return ratable_result(
-                (1,2),
-                'Conventions',
-                ["Attr Conventions does not contain 'ACDD-{}'".format(
-                            self._cc_spec_version)])
+        messages = [
+            "Attr Conventions does not contain 'ACDD-{}'".format(self._cc_spec_version)
+        ]
+        return ratable_result((1, 2),
+                              'Conventions',
+                              messages)
 
 
 class ACDDNCCheck(BaseNCCheck, ACDDBaseCheck):
     pass
 
+
 class ACDD1_1Check(ACDDNCCheck):
 
     _cc_spec_version = '1.1'
@@ -487,16 +531,25 @@ class ACDD1_1Check(ACDDNCCheck):
 
     def __init__(self):
         super(ACDD1_1Check, self).__init__()
-        self.rec_atts.extend(['keywords_vocabulary',
-                        ('cdm_data_type', ['Grid', 'Image', 'Point',
-                                           'Radial', 'Station', 'Swath',
-                                           'Trajectory'])])
+        self.rec_atts.extend([
+            'keywords_vocabulary',
+            ('cdm_data_type', [
+                'Grid',
+                'Image',
+                'Point',
+                'Radial',
+                'Station',
+                'Swath',
+                'Trajectory'
+            ])
+        ])
 
-        self.sug_atts.extend(['publisher_name',       # publisher,dataCenter
-                                'publisher_url',        # publisher
-                                'publisher_email',      # publisher
-                                'geospatial_vertical_positive'
-                              ])
+        self.sug_atts.extend([
+            'publisher_name',       # publisher,dataCenter
+            'publisher_url',        # publisher
+            'publisher_email',      # publisher
+            'geospatial_vertical_positive'
+        ])
 
 
 class ACDD1_3Check(ACDDNCCheck):
@@ -507,8 +560,9 @@ class ACDD1_3Check(ACDDNCCheck):
 
     def __init__(self):
         super(ACDD1_3Check, self).__init__()
-        self.high_rec_atts.extend([('Conventions',
-                                    self.verify_convention_version)])
+        self.high_rec_atts.extend([
+            ('Conventions', self.verify_convention_version)
+        ])
 
         self.rec_atts.extend(['geospatial_vertical_positive',
                               'geospatial_bounds_crs',
@@ -518,110 +572,110 @@ class ACDD1_3Check(ACDDNCCheck):
                               'publisher_email',      # publisher
                               'source'])
 
-        self.sug_atts.extend([  # 1.3.1, technically
-                ('creator_type', ['person', 'group', 'institution',
-                                  'position']),
-                'creator_institution',
-                ('cdm_data_type', ['Grid', 'Image', 'Point', 'Radial',
-                                   'Station', 'Swath', 'Trajectory']),
-                'platform',
-                # TODO: make dependent on platform
-                'platform_vocabulary',
-                'keywords_vocabulary',
-                'instrument',
-                'metadata_link',
-                'product_version',
-                'references',
-                ('publisher_type', ['person', 'group', 'institution',
-                                    'position']),
-                'instrument_vocabulary',
-                'date_metadata_modified',
-                'program',
-                'publisher_institution',
-            ])
-
-    def check_platform_uses_vocab(self, ds):
-        #Checks if platform vocab is in vocab list
-        if not hasattr(ds, u'platform'):
-            return
-        if not hasattr(ds, u'platform_vocabulary'):
-            return
-        platform_names_good = [platform for platform in getattr(ds, u'platform') if platform in getattr(ds, u'platform_vocabulary')]
-        return Result(BaseCheck.LOW, (len(platform_names_good),len(getattr(ds,'platform_uses_vocabulary'))), 'platforms_valid', msgs = [u'Platform_vocabulary not present'])
-
-    def check_instrument_uses_vocab(self, ds):
-        #Checks if instrument vocab is in vocab list
-        if not hasattr(ds, u'instrument'):
-            return
-        if not hasattr(ds, u'instrument_vocabulary'):
-            return
-        instrument_names_good = [instrument for instrument in
-                                 getattr(ds, u'instrument') if instrument
-                                 in getattr(ds, u'instrument_vocabulary')]
-        return Result(BaseCheck.LOW, (len(instrument_names_good),
-                      len(getattr(ds,'instrument_uses_vocabulary'))),
-                      'instruments_valid',
-                      msgs=[u'Instrument_vocabulary not present'])
+        self.sug_atts.extend([
+            ('creator_type', [
+                'person',
+                'group',
+                'institution',
+                'position'
+            ]),
+            'creator_institution',
+            ('cdm_data_type', [
+                'Grid',
+                'Image',
+                'Point',
+                'Radial',
+                'Station',
+                'Swath',
+                'Trajectory'
+            ]),
+            'platform',
+            'platform_vocabulary',
+            'keywords_vocabulary',
+            'instrument',
+            'metadata_link',
+            'product_version',
+            'references',
+            ('publisher_type', [
+                'person',
+                'group',
+                'institution',
+                'position'
+            ]),
+            'instrument_vocabulary',
+            'date_metadata_modified',
+            'program',
+            'publisher_institution',
+        ])
 
     def check_metadata_link(self, ds):
-        #Checks if metadata link is formed in a rational manner
+        '''
+        Checks if metadata link is formed in a rational manner
+
+        :param netCDF4.Dataset ds: An open netCDF dataset
+        '''
         if not hasattr(ds, u'metadata_link'):
             return
         msgs = []
         meta_link = getattr(ds, 'metadata_link')
-        if not 'http' in meta_link:
+        if 'http' not in meta_link:
             msgs.append('Metadata URL should include http:// or https://')
-        if not '.' in meta_link:
-            msgs.append('Metadata URL is malformed')
-        valid_link = len(msgs) == 0
-        return Result(BaseCheck.LOW, valid_link,  'metadata_link_valid', msgs)
+        valid_link = (len(msgs) == 0)
+        return Result(BaseCheck.LOW, valid_link, 'metadata_link_valid', msgs)
 
     def check_date_modified_is_iso(self, ds):
-        #Checks if date modified field is ISO compliant
+        '''
+        Checks if date modified field is ISO compliant
+
+        :param netCDF4.Dataset ds: An open netCDF dataset
+        '''
         if not hasattr(ds, u'date_modified'):
             return
         date_modified_check, msgs = datetime_is_iso(getattr(ds, u'date_modified'))
         return Result(BaseCheck.MEDIUM, date_modified_check, 'date_modified_is_iso', msgs)
 
     def check_date_issued_is_iso(self, ds):
-        #Checks if date issued field is ISO compliant
+        '''
+        Checks if date issued field is ISO compliant
+
+        :param netCDF4.Dataset ds: An open netCDF dataset
+        '''
         if not hasattr(ds, u'date_issued'):
             return
         date_issued_check, msgs = datetime_is_iso(getattr(ds, u'date_issued'))
         return Result(BaseCheck.MEDIUM, date_issued_check, 'date_issued_is_iso', msgs)
 
     def check_date_metadata_modified_is_iso(self, ds):
-        #Checks if date metadata modified field is ISO compliant
+        '''
+        Checks if date metadata modified field is ISO compliant
+
+        :param netCDF4.Dataset ds: An open netCDF dataset
+        '''
         if not hasattr(ds, u'date_metadata_modified'):
             return
         date_metadata_modified_check, msgs = datetime_is_iso(getattr(ds, u'date_metadata_modified'))
         return Result(BaseCheck.MEDIUM, date_metadata_modified_check, 'date_metadata_modified_is_iso', msgs)
 
     def check_id_has_no_blanks(self, ds):
-        #Check if there are blanks in the id field
+        '''
+        Check if there are blanks in the id field
+
+        :param netCDF4.Dataset ds: An open netCDF dataset
+        '''
         if not hasattr(ds, u'id'):
             return
         if ' ' in getattr(ds, u'id'):
-            return Result(BaseCheck.MEDIUM, False, 'no_blanks_in_id', msgs = [u'There should be no blanks in the id field'])
+            return Result(BaseCheck.MEDIUM, False, 'no_blanks_in_id',
+                          msgs=[u'There should be no blanks in the id field'])
         else:
-            return Result(BaseCheck.MEDIUM, True, 'no_blanks_in_id', msgs = [])
-
-    # BWA: can describe license in plain text
-    #def check_license(self, ds):
-    #    #Checks if license is from accepted list
-    #    if not hasattr(ds, u'license'):
-    #        return
-    #    license_list = {'none', 'freely distributed'}
-    #    if getattr(ds, u'license').lower() in license_list:
-    #        return Result(BaseCheck.MEDIUM, True, 'valid_license', msgs=['The license is valid'])
-    #    elif '.' in getattr(ds, u'license'):
-    #        return Result(BaseCheck.MEDIUM, True, 'valid_license', msgs=['The license is a url'])
-    #    else:
-    #        return Result(BaseCheck.MEDIUM, False, 'valid_license', msgs=['The license is not a url or in the accepted list'])
-
+            return Result(BaseCheck.MEDIUM, True, 'no_blanks_in_id', msgs=[])
 
     def check_date_created(self, ds):
-        #Check if date created is ISO
+        '''
+        Check if date created is ISO-8601
+
+        :param netCDF4.Dataset ds: An open netCDF dataset
+        '''
         if not hasattr(ds, u'date_created'):
             return
         date_created_check, msgs = datetime_is_iso(getattr(ds, u'date_created'))
@@ -630,12 +684,14 @@ class ACDD1_3Check(ACDDNCCheck):
 
     @score_group('varattr')
     def check_var_coverage_content_type(self, ds):
+        '''
+        Check coverage content type against valid ISO-19115-1 codes
+
+        :param netCDF4.Dataset ds: An open netCDF dataset
+        '''
         results = []
-        platform_variable_name = getattr(ds, 'platform', None)
-        for variable in ds.variables:
+        for variable in cfutil.get_geophysical_variables(ds):
             msgs = []
-            if variable in {'crs', platform_variable_name}:
-                continue
             ctype = getattr(ds.variables[variable],
                             'coverage_content_type', None)
             check = ctype is not None
@@ -643,14 +699,22 @@ class ACDD1_3Check(ACDDNCCheck):
                 msgs.append("Var %s missing attr coverage_content_type" %
                             variable)
                 results.append(Result(BaseCheck.HIGH, check,
-                                    (variable, "coverage_content_type"),
-                                    msgs))
-                return results
+                                      (variable, "coverage_content_type"),
+                                      msgs))
+                continue
+
             # ISO 19115-1 codes
-            valid_ctypes = {'image', 'thematicClassification', 'physicalMeasurement',
-                            'auxiliaryInformation', 'qualityInformation',
-                            'referenceInformation', 'modelResult', 'coordinate'}
-            if not ctype in valid_ctypes:
+            valid_ctypes = {
+                'image',
+                'thematicClassification',
+                'physicalMeasurement',
+                'auxiliaryInformation',
+                'qualityInformation',
+                'referenceInformation',
+                'modelResult',
+                'coordinate'
+            }
+            if ctype not in valid_ctypes:
                 msgs.append("Var %s does not have a coverage_content_type in %s"
                             % (variable, sorted(valid_ctypes)))
 
