@@ -15,6 +15,8 @@ from owslib.namespaces import Namespaces
 from compliance_checker import __version__
 from distutils.version import StrictVersion as V
 from lxml import etree
+import sys
+
 
 def get_namespaces():
     n = Namespaces()
@@ -91,7 +93,15 @@ class Result(object):
     Stores the checker instance and the check method that produced this result.
     """
 
-    def __init__(self, weight=BaseCheck.MEDIUM, value=None, name=None, msgs=None, children=None, checker=None, check_method=None):
+    def __init__(self,
+                 weight=BaseCheck.MEDIUM,
+                 value=None,
+                 name=None,
+                 msgs=None,
+                 children=None,
+                 checker=None,
+                 check_method=None,
+                 variable_name=None):
 
         self.weight = weight
 
@@ -109,6 +119,7 @@ class Result(object):
 
         self.checker = checker
         self.check_method = check_method
+        self.variable_name = variable_name
 
     def __repr__(self):
         ret = '{} (*{}): {}'.format(self.name, self.weight, self.value)
@@ -122,6 +133,12 @@ class Result(object):
         if len(self.children):
             ret += ' ({!s} children)'.format(len(self.children))
             ret += '\n' + pprint.pformat(self.children)
+
+        # python 2 requires repr to be an ASCII string
+        # python 3 requires repr to be a unicode string
+        if sys.version_info[0] < 3:
+            return ret.encode("utf-8")
+
         return ret
 
     def serialize(self):
@@ -140,6 +157,34 @@ class Result(object):
         return self.serialize() == other.serialize()
 
 
+class TestCtx(object):
+    '''
+    Simple struct object that holds score values and messages to compile into a result
+    '''
+    def __init__(self, category=None, description='', out_of=0, score=0,
+                 messages=None, variable=None):
+        self.category = category or BaseCheck.LOW
+        self.out_of = out_of
+        self.score = score
+        self.messages = messages or []
+        self.description = description or ''
+        self.variable = variable
+
+    def to_result(self):
+        return Result(self.category, (self.score, self.out_of), self.description, self.messages, variable_name=self.variable)
+
+    def assert_true(self, test, message):
+        '''
+        Increments score if test is true otherwise appends a message
+        '''
+        self.out_of += 1
+
+        if test:
+            self.score += 1
+        else:
+            self.messages.append(message)
+
+
 def std_check_in(dataset, name, allowed_vals):
     """
     Returns 0 if attr not present, 1 if present but not in correct value, 2 if good
